@@ -6,7 +6,11 @@ Prism.languages.aql = {
 		greedy: true
 	},
 	'string': {
-		pattern: /(["'´`])(?:(?!\1)[^\\\r\n]|\\.)*\1/,
+		pattern: /(["'])(?:(?!\1)[^\\\r\n]|\\.)*\1/,
+		greedy: true
+	},
+	'identifier': {
+		pattern: /([´`])(?:(?!\1)[^\\\r\n]|\\.)*\1/,
 		greedy: true
 	},
 	'variable': /@@?\w+/,
@@ -15,7 +19,7 @@ Prism.languages.aql = {
 			pattern: /(\bWITH\s+)COUNT(?=\s+INTO\b)/i,
 			lookbehind: true
 		},
-		/\b(?:AGGREGATE|ALL|AND|ANY|ASC|COLLECT|DESC|DISTINCT|FILTER|FOR|GRAPH|IN|INBOUND|INSERT|INTO|K_SHORTEST_PATHS|LET|LIKE|LIMIT|NONE|NOT|NULL|OR|OUTBOUND|REMOVE|REPLACE|RETURN|SHORTEST_PATH|SORT|UPDATE|UPSERT|WITH)\b/i,
+		/\b(?:AGGREGATE|ALL|AND|ANY|ASC|COLLECT|DESC|DISTINCT|FILTER|FOR|GRAPH|IN|INBOUND|INSERT|INTO|K_PATHS|K_SHORTEST_PATHS|LET|LIKE|LIMIT|NONE|NOT|NULL|OR|OUTBOUND|REMOVE|REPLACE|RETURN|SHORTEST_PATH|SORT|UPDATE|UPSERT|WINDOW|WITH)\b/i,
 		// pseudo keywords get a lookbehind to avoid false positives
 		{
 			pattern: /(^|[^\w.[])(?:KEEP|PRUNE|SEARCH|TO)\b/i,
@@ -26,16 +30,20 @@ Prism.languages.aql = {
 			lookbehind: true
 		},
 		{
-			pattern: /\bOPTIONS(?=\s*{)/i
+			pattern: /\bOPTIONS(?=\s*\{)/i
 		}
 	],
-	'function': /(?!\d)\w+(?=\s*\()/,
-	'boolean': /(?:true|false)/i,
+	'function': /\b(?!\d)\w+(?=\s*\()/,
+	'boolean': /\b(?:false|true)\b/i,
 	'range': {
 		pattern: /\.\./,
 		alias: 'operator'
 	},
-	'number': /(?:\B\.\d+|\b(?:0|[1-9]\d*)(?:\.\d+)?)(?:e[+-]?\d+)?/i,
+	'number': [
+		/\b0b[01]+/i,
+		/\b0x[0-9a-f]+/i,
+		/(?:\B\.\d+|\b(?:0|[1-9]\d*)(?:\.\d+)?)(?:e[+-]?\d+)?/i
+	],
 	'operator': /\*{2,}|[=!]~|[!=<>]=?|&&|\|\||[-+*/%]/,
 	'punctuation': /::|[?.:,;()[\]{}]/
 };
