@@ -47,7 +47,7 @@ Prism.languages.pcaxis = {
 		pattern: /(^|\s)\d+(?:\.\d+)?(?!\S)/,
 		lookbehind: true
 	},
-	'boolean': /YES|NO/,
+	'boolean': /NO|YES/,
 };
 
 Prism.languages.px = Prism.languages.pcaxis;
