@@ -1,19 +1,34 @@
-"use strict";
-
-const fs = require("fs");
-const { getAllFiles } = require("./test-discovery");
-const components = require("../../components.json");
-const getLoader = require("../../dependencies");
-const languagesCatalog = components.languages;
+'use strict';
+const fs = require('fs');
+const { JSDOM } = require('jsdom');
+const components = require('../../components.json');
+const getLoader = require('../../dependencies');
 const coreChecks = require('./checks');
+const { languages: languagesCatalog, plugins: pluginsCatalog } = components;
 
 
+/**
+ * @typedef {import('../../components/prism-core')} Prism
+ */
+
 /**
  * @typedef PrismLoaderContext
- * @property {import('../../components/prism-core')} Prism The Prism instance.
+ * @property {Prism} Prism The Prism instance.
  * @property {Set<string>} loaded A set of loaded components.
  */
 
+/**
+ * @typedef {import("jsdom").DOMWindow & { Prism: Prism }} PrismWindow
+ *
+ * @typedef PrismDOM
+ * @property {JSDOM} dom
+ * @property {PrismWindow} window
+ * @property {Document} document
+ * @property {Prism} Prism
+ * @property {(languages: string | string[]) => void} loadLanguages
+ * @property {(plugins: string | string[]) => void} loadPlugins
+ */
+
 /** @type {Map<string, string>} */
 const fileSourceCache = new Map();
 /** @type {() => any} */
@@ -40,6 +55,54 @@ module.exports = {
 		return context.Prism;
 	},
 
+	/**
+	 * Creates a new JavaScript DOM instance with Prism being loaded.
+	 *
+	 * @returns {PrismDOM}
+	 */
+	createPrismDOM() {
+		const dom = new JSDOM(``, {
+			runScripts: 'outside-only'
+		});
+		const window = dom.window;
+
+		window.self = window; // set self for plugins
+		window.eval(this.loadComponentSource('core'));
+
+		/** The set of loaded languages and plugins */
+		const loaded = new Set();
+
+		/**
+		 * Loads the given languages or plugins.
+		 *
+		 * @param {string | string[]} languagesOrPlugins
+		 */
+		const load = (languagesOrPlugins) => {
+			getLoader(components, toArray(languagesOrPlugins), [...loaded]).load(id => {
+				let source;
+				if (languagesCatalog[id]) {
+					source = this.loadComponentSource(id);
+				} else if (pluginsCatalog[id]) {
+					source = this.loadPluginSource(id);
+				} else {
+					throw new Error(`Language or plugin '${id}' not found.`);
+				}
+
+				window.eval(source);
+				loaded.add(id);
+			});
+		};
+
+		return {
+			dom,
+			window: /** @type {PrismWindow} */ (window),
+			document: window.document,
+			Prism: window.Prism,
+			loadLanguages: load,
+			loadPlugins: load,
+		};
+	},
+
 	/**
 	 * Loads the given languages and appends the config to the given Prism object.
 	 *
@@ -49,11 +112,7 @@ module.exports = {
 	 * @returns {PrismLoaderContext}
 	 */
 	loadLanguages(languages, context) {
-		if (typeof languages === 'string') {
-			languages = [languages];
-		}
-
-		getLoader(components, languages, [...context.loaded]).load(id => {
+		getLoader(components, toArray(languages), [...context.loaded]).load(id => {
 			if (!languagesCatalog[id]) {
 				throw new Error(`Language '${id}' not found.`);
 			}
@@ -83,7 +142,7 @@ module.exports = {
 	 */
 	createEmptyPrism() {
 		if (!coreSupplierFunction) {
-			const source = this.loadComponentSource("core");
+			const source = this.loadComponentSource('core');
 			// Core exports itself in 2 ways:
 			//  1) it uses `module.exports = Prism` which what we'll use
 			//  2) it uses `global.Prism = Prism` which we want to sabotage to prevent leaking
@@ -110,7 +169,18 @@ module.exports = {
 	 * @returns {string}
 	 */
 	loadComponentSource(name) {
-		return this.loadFileSource(__dirname + "/../../components/prism-" + name + ".js");
+		return this.loadFileSource(__dirname + '/../../components/prism-' + name + '.js');
+	},
+
+	/**
+	 * Loads the given plugin's file source as string
+	 *
+	 * @private
+	 * @param {string} name
+	 * @returns {string}
+	 */
+	loadPluginSource(name) {
+		return this.loadFileSource(`${__dirname}/../../plugins/${name}/prism-${name}.js`);
 	},
 
 	/**
@@ -123,8 +193,19 @@ module.exports = {
 	loadFileSource(src) {
 		let content = fileSourceCache.get(src);
 		if (content === undefined) {
-			fileSourceCache.set(src, content = fs.readFileSync(src, "utf8"));
+			fileSourceCache.set(src, content = fs.readFileSync(src, 'utf8'));
 		}
 		return content;
 	}
 };
+
+/**
+ * Wraps the given value in an array if it's not an array already.
+ *
+ * @param {T[] | T} value
+ * @returns {T[]}
+ * @template T
+ */
+function toArray(value) {
+	return Array.isArray(value) ? value : [value];
+}
